/*----------------------------------------------------------------------------*/
/*	(C) Copyright IBM Corporation 2012.                                   */
/*----------------------------------------------------------------------------*/
//       Example program to show the use of the "wismp" routine for
//       solving a real usymmetric system using the SSOR preconditioner.

#include <stdio.h>

extern void wismp(int *, int *, int *, double *,  double *, int *, double *, 
                  int *, int *, double *, double *, int *, double *);
extern double wsmprtc();

int main(int argc,char **argv)
{
        int n, ldb, ldx, ldy, nrhs;
        int iparm[64];
        //int ia[10];
        //int ja[53];
        double dparm[64];
        //double avals[53];
	//double x[9];
        double b[9], y[9];

        int i, nthd;
        double derr, rmisc, tt1, tt2;

//C.. Fill all arrays containing matrix data.

        n = 9; ldb = 9; ldx = 9; ldy = 9; nrhs = 1;
        int ia[] = {0, 4, 9, 15, 20, 25, 32, 38, 47, 53};

        int ja[]  = {
     0,          2,                      6,    7,
     0,    1,    2,                            7,    8,
     0,    1,    2,                      6,    7,    8,
     0, 	       3,          5,    6,    7,
     	   1,                4,    5,          7,    8,
     		 2,    3,    4,    5,    6,    7,    8,
     0,          2,    3,          5,    6,    7,
     0,    1,    2,    3,    4,    5,    6,    7,    8,
           1,    2,          4,    5,          7,    8
		 };

     double avals[] = { 
     14.0,       -1.0,                   -1.0, -3.0,
      9.0, 14.0, -1.0,                         -3.0, -1.0,
     -1.0, -1.0, 16.0,                   -2.0, -4.0, -2.0,
      8.0,             14.0,       -1.0, -1.0, -3.0,
            7.0,             14.0, -1.0,       -3.0, -1.0,
                  6.0, -1.0, -1.0, 16.0, -2.0, -4.0, -2.0,
     -1.0,       -2.0, -1.0,       -2.0, 16.0, -4.0,
     -3.0, -3.0, -4.0, -3.0, -3.0, -4.0, -4.0, 71.0, -4.0,
           -1.0, -2.0,       -1.0, -2.0,       -4.0, 16.0
		 };

      double x[] = {1.1,1.2,1.3,1.4,1.5,1.6,1.7,1.8,1.9};

//C.. Executable part of the program starts here.

//C.. Fill 'iparm' and 'dparm' arrays with default values.

        nthd = 2;	// 		! Use 2 threads
        wsetmaxthrds_(&nthd);

        iparm[0] = 0;
        iparm[1] = 0;
        iparm[2] = 0;
        wismp_(&n, ia, ja, avals, b, &ldb, x, &ldx, &nrhs, &rmisc, NULL, iparm, 
               dparm);
        if (iparm[63] != 0) {
          printf("The following ERROR was detected in S0: %d", iparm[63]);
          exit(1);
        }

//C.. Modify some defaults

	iparm[4] = 0;  //	 	! C-style numbering
	iparm[14] = 2;  //	 	! SSOR preconditioning

//C.. Analyze structure

	tt1 = wsmprtc();
        iparm[1] = 1;
        iparm[2] = 1;
        wismp_(&n, ia, ja, avals, b, &ldb, x, &ldx, &nrhs, &rmisc, NULL, iparm, 
               dparm);
        if (iparm[63] != 0) {
          printf("The following ERROR was detected in S1: %d",iparm[63]);
          exit(1);
        }
	tt2 = wsmprtc();
        printf("Step 1 complete in time - %g\n",tt2-tt1);

//C.. Analyze Values

        tt1 = wsmprtc();
        iparm[1] = 2;
        iparm[2] = 2;
        wismp_(&n, ia, ja, avals, b, &ldb, x, &ldx, &nrhs, &rmisc, NULL, iparm, 
               dparm);
        if (iparm[63] != 0) {
          printf("The following ERROR was detected in S2: %d",iparm[63]);
          exit(1);
        }
        tt2 = wsmprtc();
        printf("Step 2 complete in time - %g\n",tt2-tt1);

//C.. Matrix-Vector multiplication (compute b = Ax).

        tt1 = wsmprtc();
        iparm[1] = 5;
        iparm[2] = 5;
        wismp_(&n, ia, ja, avals, b, &ldb, x, &ldx, &nrhs, &rmisc, NULL, iparm, 
               dparm);
        if (iparm[63] != 0) {
          printf("The following ERROR was detected in S5: %d", iparm[63]);
          exit(1);
        }
        tt2 = wsmprtc();
        printf("Step 5 complete in time - %g\n",tt2-tt1);

//C.. Generate Preconditioner.

        tt1 = wsmprtc();
        iparm[1] = 3;
        iparm[2] = 3;
        wismp_(&n, ia, ja, avals, b, &ldb, x, &ldx, &nrhs, &rmisc, NULL, iparm,
               dparm);
        if (iparm[63] != 0) {
          printf("The following ERROR was detected in S3: %d", iparm[63]);
          exit(1);
        }
        tt2 = wsmprtc();
        printf("Step 3 complete in time - %g\n",tt2-tt1);

//C.. Iterative Solution (solve for y in Ay = b).

        tt1 = wsmprtc();
        iparm[1] = 4;
        iparm[2] = 4;
        wismp_(&n, ia, ja, avals, b, &ldb, y, &ldy, &nrhs, &rmisc, NULL, iparm,
               dparm);
        if (iparm[63] != 0) {
          printf("The following ERROR was detected in S4: %d", iparm[63]);
          exit(1);
        }
        tt2 = wsmprtc();
        printf("Step 4 complete in %g seconds in %d iters\n",tt2-tt1,iparm[25]);

	rmisc = 0.0;
	derr = 0.0;
        for (i = 0; i < n; i++) {
	  printf("%d : %g\n",i,y[i]);
	  derr = derr + (y[i] - x[i]) * (y[i] - x[i]);
          rmisc = rmisc + b[i] * b[i];
	}
        printf("Residual = %g; Relative error = %g\n",dparm[25],derr/rmisc);

	return 0;
}

